/* ----------------------------------------------------------------------------
%                            Team Project "lejON"
%
% master study course at the
% University of Applied Studies and Research in Wernigerode, Germany (HS Harz)
% in winter 2005/06
% -----------------------------------------------------------------------------
%
%
% File:           FungousParcours.java 
% Initial author: Kai Schories, April 2006
% Latest Autor:   Florian Ruh
% 
% -----------------------------------------------------------------------------
*/

import josx.platform.rcx.Motor;
import josx.platform.rcx.Sensor;

import org.lejon.DDModel;
import org.lejon.events.MotionCommand;
import org.lejon.events.MotionReport;
import org.lejon.events.MotionEventListener;
import org.lejon.util.DriveParameters;
import org.lejon.util.MoveMentor;


/**
 * This is a lejON-related class for testing purposes.
 * It lets the robot drive a parcours that looks like the profile of a fungus:
 *
 * Drive forwards, rotate 90 deg, circle, rotate 90 deg, drive forwards.
 *
 * See how the code corrects the circle and rotation figures by using 
 * MoveMentor.rectify(). It retrieves its position by using the self-localisation 
 * methods and compares the position to the aimed one. If it is outside a certain
 * range of tolerance then the figure will be continued with the difference
 * between aimed and current position. In case of rotation, the figure must be
 * driven to the opposite direction.
 */

public class FungousParcours implements MotionEventListener
{
	private boolean isLastFigure;
  private DDModel myDrive;
	private MotionCommand mMotionCommand = new MotionCommand();
	private MotionReport mMotionReport = null;

	private boolean finished = false;

	public FungousParcours()
	{
		// experimentally defined velocity vector for DIDI 
		int [] velocities = new int [] {1, 3, 5, 5, 6, 6, 6, 6};

		//PID constants for DIDI
		float [] pidConstants = new float[] {1.56f, 0.48f, 0.8f};
                
        // set dd parameters
        DriveParameters params = new DriveParameters
        (
			16,  // ticks nominal
			10.5f,  // axle length
			7.0f,  // wheel diameter
			40.0f / 24.0f,  // gear ratio
			Motor.C.getId(), // motor ID left
			Motor.A.getId(), // motor ID right
			Sensor.S3.getId(), // rotation sensor ID left
			Sensor.S1.getId(),// rotation sensor ID right
			true, // rotation sensors count backward
			false, // wheels and motors rotate in the same direction
			pidConstants, // PID constants
			100, // time base in [ms]
			6, // maximum ticks per time base
			velocities // velocity vector
		);

		//setup Drive regarding parameters
		myDrive = DDModel.instance(params);

		//add MotionEventListener
		myDrive.addMotionEventListener(this);
	}
	
	public void doParcours(int dist, int radius)
	{
		//move ahead
		mMotionCommand.setAction(MotionCommand.TRANSLATE);
		mMotionCommand.setDistance(dist);
		mMotionCommand.setMaxPower(4);

		doFigure();

		//rotate right 90 degrees
		mMotionCommand.setAction(MotionCommand.ROTATE);
		mMotionCommand.setAngle(-90);
		mMotionCommand.setMaxPower(1);

		doFigure();

		//do a circle
		mMotionCommand.setAction(MotionCommand.CURVE);
		mMotionCommand.setRadius(radius);
		mMotionCommand.setAngle(360);
		mMotionCommand.setMaxPower(4);
	
		doFigure();

		//rotate right 90 degrees
		mMotionCommand.setAction(MotionCommand.ROTATE);
		mMotionCommand.setAngle(-90);
		mMotionCommand.setMaxPower(1);


		doFigure();

		//move ahead to origin
		mMotionCommand.setAction(MotionCommand.TRANSLATE);
		mMotionCommand.setDistance(dist);
		mMotionCommand.setMaxPower(4);


		isLastFigure = true;

		doFigure();

		return;
	}
		

	private void doFigure()
	{
		do
			{
				finished = false;

				myDrive.setMotionCommand(mMotionCommand);
				myDrive.move();

				while (! finished)
					{
						//wait for event MotionFinished
						Thread.yield();
					}

				try 
					{
						Thread.sleep(500);
					}
				catch (InterruptedException e)
					{
						
					} // end of try-catch
				
				
			}
 		while ( MoveMentor.rectify( mMotionCommand, mMotionReport ) );
		if( isLastFigure )
		{
			MoveMentor.showNumber( (int) mMotionReport.getDistance() );
		}

	}

	public void motionFinished(MotionReport report)
	{
		mMotionReport = report;
		finished = true;
		MoveMentor.showNumber( (int) report.getDistance() );

		return;
	}
		

	public static void main(String[] args)
	{
		FungousParcours parcours = new FungousParcours();
		while(true)
		{
			MoveMentor.startCounter(3);
			parcours.doParcours(150,35);
			MoveMentor.waitForRUNPressed();
		}
		

	}
}
		
		
				
		